function [ref, trs] = getCoefficientByMultilayerPlaneUnderPlaneWave(z, ...
                                                      layer, ...
                                                      background, ...
                                                      frequency, ...
                                                      varargin)
    % [ref, trs] = getCoefficientByMultilayerPlaneUnderPlaneWave(z,
    %                                                            layer, 
    %                                                            background, 
    %                                                            frequency)
    % 
    % Calculate the reflection and transmission coefficients as a
    % plane wave is scattered by a multilayer slab. The incident plane
    % wave is polarized in the +x direction propagating from the left
    % (neg Z) to the right (pos Z). The time-harmonic factor is
    % e^{j\omega t}.
    %
    %
    %   X
    %   ^
    %   |
    %   |  background       k_1     k_2     k_3     k_4        
    %   ----------------|--------|-------|-------|---------> z
    %                  z_1      z_2     z_3     z_4
    %
    %
    % Input:
    %
    % z            Mx1 vector in (m)
    %
    % layer        Mx1 cell array of objects of DielectricMaterial
    %
    % background   object of DielectricMaterial
    %
    % frequency    Nx1 vector (Hz)
    %
    % Output:
    %
    % ref          Nx(M+1) matrix. Each row contains the reflection
    %              coefficient. The last element of each row is 0,
    %              i.e. no reflection in the very last slab.
    %
    % trs          Nx(M+1) matrix. Each row contains the transmission
    %              coefficient. The first element of each row is 1.
    %
    %
    
    p = inputParser;
    p.addRequired('z', @isvector);
    p.addRequired('layer', @isvector);
    p.addRequired('bacgkround', @isobject);
    p.addRequired('frequency', @isvector);
    p.addParamValue('verbose', 0, @(x)x==0||x==1);
    p.addParamValue('debug',0, @(x)x==0||x==1);
    p.parse(z, layer, background, frequency, varargin{:});
    if (p.Results.verbose)
        disp(p.Results);
    end
    
    ref = zeros(length(frequency), length(layer)+1);
    trs = ones(length(frequency), length(layer)+1);
    for i = 1:length(frequency)
        % pre allocation
        k      = zeros(length(layer)+1,1);
        eta    = zeros(length(layer)+1,1);

        % calculate the wave number and intrinsic impedance
        k(1)   = getWaveNumber(background, frequency(i));
        eta(1) = getIntrinsicImpedance(background, frequency(i));
        for iLayer = 1:length(layer)
            k(iLayer+1)   = getWaveNumber(layer{iLayer}, frequency(i));
            eta(iLayer+1) = getIntrinsicImpedance(layer{iLayer}, frequency(i));
        end
        
        if (p.Results.verbose)
            fprintf('Cplx wave number: %f\t Cplx impedance: %f\t\n', k, eta);
        end
        
        if (length(layer) == 1)
            % calculate the transmission matrix for the interface
            A = [         exp(-1j*k(1)*z(1))             exp(+1j*k(1)*z(1));
                 1/eta(1)*exp(-1j*k(1)*z(1))   -1/eta(1)*exp(+1j*k(1)*z(1))];
            B = [         exp(-1j*k(2)*z(1))             exp(+1j*k(2)*z(1));
                 1/eta(2)*exp(-1j*k(2)*z(1))   -1/eta(2)*exp(+1j*k(2)*z(1))];
            T = inv22(B)*A;

            % apply the boundary condition
            ref(i,1) =-T(2,1)/T(2,2);
            trs(i,2) = T(1,1)-(T(1,2)*T(2,1)/T(2,2));
            
        elseif (length(layer) == 2)
            % calculate the transmission matrix at every interface
            A = [         exp(-1j*k(1)*z(1))             exp(+1j*k(1)*z(1));
                 1/eta(1)*exp(-1j*k(1)*z(1))   -1/eta(1)*exp(+1j*k(1)*z(1))];
            B = [         exp(-1j*k(2)*z(1))             exp(+1j*k(2)*z(1));
                 1/eta(2)*exp(-1j*k(2)*z(1))   -1/eta(2)*exp(+1j*k(2)*z(1))];
            T_interface(1,:,:) = inv22(B)*A;
            
            A = [         exp(-1j*k(2)*z(2))             exp(+1j*k(2)*z(2));
                 1/eta(2)*exp(-1j*k(2)*z(2))   -1/eta(2)*exp(+1j*k(2)*z(2))];
            B = [         exp(-1j*k(3)*z(2))             exp(+1j*k(3)*z(2));
                 1/eta(3)*exp(-1j*k(3)*z(2))   -1/eta(3)*exp(+1j*k(3)*z(2))];
            T_interface(2,:,:) = inv22(B)*A;
            
            % calculate the overall transmission matrix
            T = squeeze(T_interface(2,:,:))*squeeze(T_interface(1,:,:));
            
            % apply the boundary condition
            ref(i,1  ) =-T(2,1)/T(2,2);
            trs(i,end) = T(1,1)-(T(1,2)*T(2,1)/T(2,2));
            
            % calculate the transmission and reflection coefficient
            % in the intermediate layers
            b       = squeeze(T_interface(1,:,:))*[1; ref(i,1)];
            trs(i,2) = b(1);
            ref(i,2)= b(2);
            
            % use the last calculated transmission and reflection
            % coefficients to estimate the error
            x = [trs(i,end-1); ref(i,end-1)];
            b = squeeze(T_interface(end,:,:))*x;
            if (b(2) > 0.001)
                fprintf(['At f=%f, Expected ref(end) is 0, numerical results is %d\n'], ...
                        frequency(i),b(2));
                disp(['The round off error is larger than 0.001']);
            end

        else
            % calculate the transmission matrix at every interface
            T_interface = zeros(length(layer),2,2);
            for iLayer=1:length(layer);
                A = [              exp(-1j*k(iLayer)*z(iLayer))                  exp(+1j*k(iLayer)*z(iLayer));
                     1/eta(iLayer)*exp(-1j*k(iLayer)*z(iLayer))   -1/eta(iLayer)*exp(+1j*k(iLayer)*z(iLayer))];
                B = [                exp(-1j*k(iLayer+1)*z(iLayer))                    exp(+1j*k(iLayer+1)*z(iLayer));
                     1/eta(iLayer+1)*exp(-1j*k(iLayer+1)*z(iLayer))   -1/eta(iLayer+1)*exp(+1j*k(iLayer+1)*z(iLayer))];
                T_interface(iLayer,:,:)=inv22(B)*A;
            end
            
            % calculate the overall transmission matrix
            T = eye(2);
            for iLayer=1:length(layer)
                T = squeeze(T_interface(iLayer,:,:))*T;
            end
            
            % apply the boundary condition
            ref(i,1  ) =-T(2,1)/T(2,2);
            trs(i,end) = T(1,1)-(T(1,2)*T(2,1)/T(2,2));
            
            % calculate the transmission and reflection
            % coefficients in the intermediate layers
            for iLayer=1:length(layer)-1;
                x = [trs(i,iLayer); ref(i,iLayer)];
                b = squeeze(T_interface(iLayer,:,:))*x;
                trs(i,iLayer+1)=b(1);
                ref(i,iLayer+1)=b(2);
            end
            
            % use the reflection coefficient (which should be zero) of the last
            % medium to estimate the error
            x = [trs(i,end-1); ref(i,end-1)];
            b = squeeze(T_interface(end,:,:))*x;
            if (abs(b(2)) > 0.001)
                fprintf(['At f=%f, Expected ref(end) is 0, numerical results is %d\n'], ...
                        frequency(i),b(2));
                disp(['The round off error is larger than 0.001']);
            end
            
            if (p.Results.verbose)
                fprintf(['Expected ref(end) is 0, numerical result is %d\n'],...
                        abs(b(2)));
            end
            
        end %Layer
    end %iFrequency
end %getCoefficientByMultilayerPlaneUnderPlaneWave

function inv_A = inv22(A)
    [m, n] = size(A);
    assert(m==n && m==2);
    inv_A = [+A(2,2) -A(1,2)
             -A(2,1) +A(1,1)]/det(A);
end
