function [Ex, Hy] = getFieldByMultilayerPlaneUnderPlaneWave(z, ...
                                                      layer, ...
                                                      background, ...
                                                      sensor_location, ...
                                                      frequency, ....
                                                      varargin)
    % Ex = getFieldByMultilayerPlaneUnderPlaneWave(z, 
    %                                              layer, 
    %                                              background,
    %                                              sensor_location, 
    %                                              frequency) 
    %
    % Calculate the field as a plane wave is scattered by a
    % multi-layer slab. The incident plane wave is polarized in the +x
    % direction propagating from left (neg Z) to right (pos Z). The
    % time-harmonic factor is e^{j\omega t}.
    %
    % If the sensor_location is in the left most region, the function
    % returns the reflected field. If the sensor_location is in the
    % right most region, the function returns the transmitted
    % field. If the sensor_location is inside a slab, it returns the
    % total field inside the slab. 
    %
    %
    %   X
    %   ^
    %   |
    %   |  background       k_1     k_2     k_3     k_4        
    %   ----------------|--------|-------|-------|---------> z
    %                  z_1      z_2     z_3     z_4
    % 
    %
    % Input:
    %
    % z                Mx1 vector in (m)
    %
    % layer            Mx1 cell array of objects of DielectricMaterial
    %
    % background       object of DielectricMaterial
    %
    % sensor_location  scalar in (m)
    % 
    % frequency        N x 1 vector in (Hz)
    %
    % Output:
    %
    % Ex               Nx1 vector (V/m)
    % 
    % Hy               Nx1 vector (A/m)


    p = inputParser;
    p.addRequired('z', @isnumeric);
    p.addRequired('layer', @isvector);
    p.addRequired('background', @isobject);
    p.addRequired('sensor_location', @isnumeric);
    p.addRequired('frequency', @isnumeric);
    p.addParamValue('verbose', 0, @(x)x==0||x==1);
    p.parse(z, layer, background, sensor_location, frequency, varargin{:});
    if (p.Results.verbose)
        disp(p.Results);
    end
    
    [ref, trs] = getCoefficientByMultilayerPlaneUnderPlaneWave(z, ...
                                                      layer, ...
                                                      background, ...
                                                      frequency, ...
                                                      varargin{:});
    Ex = zeros(length(frequency), 1);
    Hy = zeros(length(frequency), 1);
    
    if sensor_location < z(1) % reflection
        k  = getWaveNumber(background, frequency);
        Ex = ref(:,  1).*exp(+j*k*sensor_location);
        eta= getIntrinsicImpedance(background, frequency);
        Hy = Ex./eta;
    elseif z(end) < sensor_location % transmission
        k  = getWaveNumber(layer{end}, frequency);
        Ex = trs(:,end).*exp(-j*k*sensor_location); 
        eta= getIntrinsicImpedance(layer{end}, frequency);
        Hy = Ex./eta;
    else % inside a slab
        idx= length(find(z < sensor_location));
        k  = getWaveNumber(layer{idx}, frequency);
        Ex = trs(:, idx).*exp(-j*k*sensor_location) + ...
             ref(:, idx).*exp(+j*k*sensor_location);
        eta= getIntrinsicImpedance(layer{idx}, frequency);
        Hy = Ex./eta;
    end
end

