function Balanis1989Transformer(varargin)
% Responses of singlesection, two-section binomial, and two-section
% Tschebyscheff quarterwavelength transformers.
%
% Duplicate Fig. 5-19 in [Balanis1989]. See Example 5-11 on pg. 232.
%
%
    addpath('../src');
    
    background = DielectricMaterial(1, 0);
    f_0        = 10e9;
    f          = linspace(0, 20e9, 256)';
    theta = pi/2*(f/f_0);
    
    % one-section
    layer = {DielectricMaterial(2, 0),
             DielectricMaterial(4, 0)};
    z     = [0; 
             getWavelength(DielectricMaterial(2,0), f_0)/4];
    ref   = getCoefficientByMultilayerPlaneUnderPlaneWave(z, layer, background, f);
    ref_1 = ref(:,1);

    % two-section binomial design, analytic expression in (5-76)
    ref_2 = (getIntrinsicImpedance(DielectricMaterial(4.0, 0), f_0) - getIntrinsicImpedance(background, f_0)) ...
            /(getIntrinsicImpedance(DielectricMaterial(4.0, 0), f_0) + getIntrinsicImpedance(background, f_0)) ...
            *exp(-j*2*theta) ...
            .*cos(theta).^2;

    % two-section binomial design, numerical
    layer = {DielectricMaterial(1.4, 0),
             DielectricMaterial(2.74,0),
             DielectricMaterial(4,   0)};
    z     = [0; 
             getWavelength(DielectricMaterial(1.40,0), f_0)/4;
             getWavelength(DielectricMaterial(2.74,0), f_0)/4];
    z     = cumsum(z);
    ref   = getCoefficientByMultilayerPlaneUnderPlaneWave(z, layer, background, f);
    ref_4 = ref(:,1);
    
    % two-section Tschebyscheff design, analytic expression in (5-81)
    theta_m = (1-(1/2)*0.375)*pi/2; % in (5-78)
    ref_5 = (getIntrinsicImpedance(DielectricMaterial(4.0, 0), f_0) - getIntrinsicImpedance(background, f_0)) ...
            /(getIntrinsicImpedance(DielectricMaterial(4.0, 0), f_0) + getIntrinsicImpedance(background, f_0)) ...
            *exp(-j*2*theta) ...
            .*TschebyscheffPoly2nd(sec(theta_m)*cos(theta))./TschebyscheffPoly2nd(sec(theta_m));

    figure; 
    set(gcf, 'color', 'white');
    plot(f/f_0, abs(ref_1), 'b-', 'linewidth', 2); 
    hold('on');
    plot(f/f_0, abs(ref_2), 'r-.', 'linewidth', 2);
    plot(f/f_0, abs(ref_4), 'g-', 'linewidth', 2);
    
    plot(f/f_0, abs(ref_5), 'm-', 'linewidth', 2);
    
    set(gca, 'fontname', 'arial', 'fontsize', 12);
    xlabel('Relative frequency (f/f_0)');
    ylabel('Input reflection coefficient (\Gamma_{in})');
    legend('One-section', ...
           'Two-section binomial (analytical)', ...
           'Two-section binomial (numerical)', ...           
           'Two-section Tschebyscheff (analytical)', ...
           'location', 'northeast');

end

function data = TschebyscheffPoly2nd(x)
% The second-order Chebyshev polynomial
    data = 2*x.^2 - 1;
end
